from Student import Student


class StudentsAdministrationBSTNode:
    def __init__(self, key, val):
        self.key = key  # a simple integer instead of date for simplification
        self.val = val
        self.left = None
        self.right = None

    def __repr__(self):
        return str(self.key) + ": " + str(self.val)

    def is_leaf(self):
        return (self.left is None) and (self.right is None)


class StudentsAdministrationBST:
    def __init__(self):
        self.root = None

    def __repr__(self):
        # a simplified repr:
        return 'This is the students administration data structure'

    def insert(self, key, val):
        """ insert node with key,val into tree, uses recursion """

        def insert_rec(node, key, val):
            if key == node.key:
                node.val = val  # update the val for this key
            elif key < node.key:  # left
                if node.left == None:
                    node.left = StudentsAdministrationBSTNode(key, val)
                else:
                    insert_rec(node.left, key, val)
            else:  # key > node.key:
                if node.right == None:
                    node.right = StudentsAdministrationBSTNode(key, val)
                else:
                    insert_rec(node.right, key, val)
            return

        if self.root is None:  # empty tree
            self.root = StudentsAdministrationBSTNode(key, val)
        else:
            insert_rec(self.root, key, val)

    def inorder(self):
        """ returns inorder traversal of key value pairs as tuples, uses recursion """

        def inorder_rec(curr_node, res):
            if curr_node is not None:
                inorder_rec(curr_node.left, res)
                res.append((curr_node.key, curr_node.val))
                inorder_rec(curr_node.right, res)
            return res

        if self.root is None:  # empty tree
            return "Empty Tree"
        else:
            return inorder_rec(self.root, [])

    def reversed_in_order(self):
        """
        returns reversed in-order traversal of key value pairs as tuples, uses recursion
        reversed post order is: right,left,center (instead of left,right,center).
        """

        def reversed_in_order_rec(curr_node, res):
            if curr_node is not None:
                reversed_in_order_rec(curr_node.right, res)
                res.append((curr_node.key, curr_node.val))
                reversed_in_order_rec(curr_node.left, res)
            return res

        if self.root is None:  # empty tree
            return "Empty Tree"
        else:
            return reversed_in_order_rec(self.root, [])

    def print_grades(self):
        all_students_by_in_order = self.inorder()
        res = 'All students grades:\n'
        for enrolment, student_obj in all_students_by_in_order:
            res += f"{student_obj.name}: average grade = {student_obj.avg_grade}\n"
        return res

    def print_name_by_seniority(self):
        all_students_by_in_order = self.reversed_in_order()
        res = 'All students name by seniority:\n'
        for enrolment, student_obj in all_students_by_in_order:
            res += f"Enrolled at {enrolment}: name = {student_obj.name}\n"
        return res


if __name__ == '__main__':
    # # creating the tree
    bgu_administration_ds = StudentsAdministrationBST()
    # This is an example for how bad testing might trick you:
    students = [Student(name=f'{"A" * idx}', avg_grade=100 - idx) for idx in range(1, 11)]

    # instead of real date, we will use a simple integer index
    for idx, student_obj in enumerate(students):
        bgu_administration_ds.insert(key=idx, val=student_obj)

    # Section A
    print(bgu_administration_ds.print_grades())
    # Section B
    print(bgu_administration_ds.print_name_by_seniority())

    # and this is how we catch this mistake
    students = [Student("name_9", 20), Student("name_2", 50), Student("name_5", 80), Student("name_3", 76),
                Student("name_1", 100), Student("name_4", 39), Student("name_6", 99), Student("name_7", 95),
                Student("name_8", 5)]
    # instead of real date, we will use a simple integer index
    index = [9, 2, 5, 3, 1, 4, 6, 7, 8]
    for idx, student_obj in enumerate(students):
        bgu_administration_ds.insert(key=index[idx], val=student_obj)

    # Section A
    print(bgu_administration_ds.print_grades())
    # Section B
    print(bgu_administration_ds.print_name_by_seniority())